/*
    $File$
    Date        Author          Changes
    Nov 21 2000 Richard Jones   Created
 */

 
package MetaTF.Parser;

import java.util.*;
import MetaTF.Panic;

/**
 * Field Lister visitor
 * adds a list of fields the symbol table in the order that they appear, with no duplicates.
 *
 * @see MetaTF.Parser.Field
 * @version rel-1-2 ($Revision: 1.2 $, $Date: 2000/11/29 18:06:08 $),
 * @author Richard Jones
**/
public class FieldLister implements MetaTFParserVisitor {
  private static final String cvsheader = "$Header: /home/cur/rej/cvs/MetaTF/src/MetaTF/Parser/FieldLister.java,v 1.2 2000/11/29 18:06:08 rej Exp $";

  public Object visit(SimpleNode node, Object data) {
    Panic.panic("FieldLister found a SimpleNode");
    return null;
  }

  public Object visit(ASTDTD node, Object symtab) {
    List fields = new LinkedList();
    for(int i=0; i < node.jjtGetNumChildren(); i++) 
        append(fields, (List)node.jjtGetChild(i).jjtAccept(this, null));
    ((DTD)symtab).addFields(fields);
    return fields;
  }

  public Object visit(ASTSection0 node, Object data) {
    List fields = new LinkedList();
    for(int i=0; i < node.getNumPropertiesRecords(); i++) 
        append(fields, (List)node.getPropertyOrRecord(i).jjtAccept(this, null));
    return fields;
  }

  public Object visit(ASTSection node, Object data) {
    List fields = new LinkedList();
    for(int i=2; i < node.getNumPropertiesRecords(); i++) 
        append(fields, (List)node.getPropertyOrRecord(i).jjtAccept(this, null));
    return fields;
  }

  // There's a question here of which field names we should add?
  // All, only properly defined ones, names other than fields,... ??
  
  public Object visit(ASTRecord node, Object data) {
    List fields = new LinkedList();
    append(fields, (List)node.getRecordField().jjtAccept(this, node));  
    for(int i=0; i < node.getNumProperties(); i++) 
        append(fields, (List)node.getProperty(i).jjtAccept(this, null));
    return fields;
  }

  public Object visit(ASTNamedField node, Object data) {
    List fields = new LinkedList();
    fields.add(node.getName());
    return fields;
  }

  public Object visit(ASTRptField node, Object data) {
    List fields = new LinkedList();
    fields.add(node.getRepeat().getName());
    append(fields, (List)node.getRepeated().jjtAccept(this,data));
    return fields;
  }

  public Object visit(ASTTupleField node, Object data) {
    List fields = new LinkedList();
    for(int i=0; i<node.getArity(); i++) 
        append(fields, (List)node.getField(i).jjtAccept(this, data));
    return fields;
  }

  public Object visit(ASTProperty node, Object data) {
    List fields = new LinkedList();
    fields.add(node.getFieldName());
    return fields;
  }

  public Object visit(ASTName node, Object data) {
    List fields = new LinkedList();
    fields.add(node.getName());
    return fields;
  }


  // NOT USED
  public Object visit(ASTNone node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTNone");
    return null;
  }

  public Object visit(ASTBaseOffset node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTBaseOffset");
    return null;
  }

  public Object visit(ASTDelta node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTDelta");
    return null;
  }

  public Object visit(ASTDefault node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTDefault");
    return null;
  }

  public Object visit(ASTStringDefault node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTStringDefault");
    return null;
  }

  public Object visit(ASTSectionOffset node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTSectionOffset");
    return null;
  }

  public Object visit(ASTStride node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTStride");
    return null;
  }

  public Object visit(ASTSectionStride node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTSectionStride");
    return null;
  }

  public Object visit(ASTNumberValue node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTNumberValue");
    return null;
  }

  public Object visit(ASTNumber node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTNumber "+node.getNumber());
    return null;
  }

  public Object visit(ASTTraceString node, Object data) {
    node.dump("Panic: ");
    Panic.panic("FieldLister finds an ASTTraceString "+node.getString());
    return null;
  }

  /**
   * Append lists without adding duplicates
   *
   * @param list1 the initial list
   * @param list2 the list to be appended
   * @return list1
  **/
  private List append(List list1, List list2) {
    Iterator it2 = list2.iterator();
    while(it2.hasNext()) {
        Object o = it2.next();
        if(!list1.contains(o))
            list1.add(o);
    }
    return list1;
  }
}
